@@ -72,8 +72,10 @@ void *thread_function(void *arg) {
   igraph_real_t *v, *workl, *workd, *d, *resid, *ax;
   int *select;
   int ido=0;
+#if IGRAPH_THREAD_SAFE
   int rvec=1;
   char *all="All";
+#endif
   int i;
 
   igraph_arpack_options_init(&options);
@@ -105,10 +107,12 @@ void *thread_function(void *arg) {
   }
   
   while (1) {
+#if IGRAPH_THREAD_SAFE
     igraphdsaupd_(&ido, options.bmat, &options.n, options.which,
 		  &options.nev, &options.tol, resid, &options.ncv, v, 
 		  &options.ldv, options.iparam, options.ipntr, workd,
 		  workl, &options.lworkl, &options.info);
+#endif
     
     if (ido == -1 || ido == 1) {
       
@@ -142,12 +146,14 @@ void *thread_function(void *arg) {
     return 0;
   }
 
+#if IGRAPH_THREAD_SAFE
   igraphdseupd_(&rvec, all, select, d, v, &options.ldv,
 		&options.sigma, options.bmat, &options.n,
 		options.which, &options.nev, &options.tol,
 		resid, &options.ncv, v, &options.ldv, options.iparam,
 		options.ipntr, workd, workl, &options.lworkl,
 		&options.ierr);
+#endif
   
   if (options.ierr != 0) {
     printf("ARPACK error\n");
