@@ -24,13 +24,23 @@
 #ifndef IGRAPH_VERSION_H
 #define IGRAPH_VERSION_H
 
-#define IGRAPH_VERSION "0.6.5"
+#include "igraph_decls.h"
+
+__BEGIN_DECLS
+
+#define IGRAPH_VERSION "0.7.1999"
+#define IGRAPH_VERSION_MAJOR 0
+#define IGRAPH_VERSION_MINOR 7
+#define IGRAPH_VERSION_PATCH 1999
+#define IGRAPH_VERSION_PRERELEASE ""
 
 int igraph_version(const char **version_string,
 		   int *major,
 		   int *minor,
 		   int *subminor);
 
+__END_DECLS
+
 #endif
 
 
