@@ -157,11 +157,10 @@ int igraph_empty_attrs(igraph_t *graph, igraph_integer_t n, igraph_bool_t direct
  * iterators of a graph should be destroyed before the graph itself
  * anyway. 
  * \param graph Pointer to the graph to free.
- * \return Error code.
  * 
  * Time complexity: operating system specific.
  */
-int igraph_destroy(igraph_t *graph) {
+void igraph_destroy(igraph_t *graph) {
 
   IGRAPH_I_ATTRIBUTE_DESTROY(graph);
 
@@ -170,9 +169,7 @@ int igraph_destroy(igraph_t *graph) {
   igraph_vector_destroy(&graph->oi);
   igraph_vector_destroy(&graph->ii);
   igraph_vector_destroy(&graph->os);
-  igraph_vector_destroy(&graph->is);
-  
-  return 0;
+  igraph_vector_destroy(&graph->is);  
 }
 
 /**
@@ -1030,15 +1027,14 @@ int igraph_degree(const igraph_t *graph, igraph_vector_t *res,
 
 int igraph_edge(const igraph_t *graph, igraph_integer_t eid, 
 		igraph_integer_t *from, igraph_integer_t *to) {
-  
-  *from = (igraph_integer_t) VECTOR(graph->from)[(long int)eid];
-  *to   = (igraph_integer_t) VECTOR(graph->to  )[(long int)eid];
-  
-  if (! igraph_is_directed(graph) && *from > *to) {
-    igraph_integer_t tmp=*from;
-    *from=*to;
-    *to=tmp;
-  }
+
+  if (igraph_is_directed(graph)) {
+    *from = (igraph_integer_t) VECTOR(graph->from)[(long int)eid];
+    *to   = (igraph_integer_t) VECTOR(graph->to  )[(long int)eid];
+  } else {
+    *from = (igraph_integer_t) VECTOR(graph->to  )[(long int)eid];
+    *to   = (igraph_integer_t) VECTOR(graph->from)[(long int)eid];
+  }    
 
   return 0;
 }
@@ -1053,10 +1049,18 @@ int igraph_edges(const igraph_t *graph, igraph_es_t eids,
   IGRAPH_FINALLY(igraph_eit_destroy, &eit);
   n=IGRAPH_EIT_SIZE(eit);
   IGRAPH_CHECK(igraph_vector_resize(edges, n*2));
-  for (; !IGRAPH_EIT_END(eit); IGRAPH_EIT_NEXT(eit)) {
-    long int e=IGRAPH_EIT_GET(eit);
-    VECTOR(*edges)[ptr++]=IGRAPH_FROM(graph, e);
-    VECTOR(*edges)[ptr++]=IGRAPH_TO(graph, e);
+  if (igraph_is_directed(graph)) {
+    for (; !IGRAPH_EIT_END(eit); IGRAPH_EIT_NEXT(eit)) {
+      long int e=IGRAPH_EIT_GET(eit);
+      VECTOR(*edges)[ptr++]=IGRAPH_FROM(graph, e);
+      VECTOR(*edges)[ptr++]=IGRAPH_TO(graph, e);
+    }
+  } else {
+    for (; !IGRAPH_EIT_END(eit); IGRAPH_EIT_NEXT(eit)) {
+      long int e=IGRAPH_EIT_GET(eit);
+      VECTOR(*edges)[ptr++]=IGRAPH_TO(graph, e);
+      VECTOR(*edges)[ptr++]=IGRAPH_FROM(graph, e);
+    }
   }
   
   igraph_eit_destroy(&eit);
